### Main client game module

import pygame
from pygame.locals import *

import client
import interface
import dm_canvas
dm_canvas.load_tiles()
import server_stub
import sound
from sound import sounds
import random
import things

class DMInterface(interface.Interface):

    def __init__(self,position=(0,0)):
        interface.Interface.__init__(self)
        self.canvas = dm_canvas.Canvas(position)

        self.selected_thing = None

        logo = pygame.image.load('images/pq.bmp').convert()
        logo.set_colorkey((199,0,0))

        # create side bar panel
        sp = dm_canvas.sidePanel(client.panelsize,client.screenheight)
        sp.set_background((220,220,200))
        sp.replace_top(logo)

        self.sp = sp

    def update_everything(self):
        self.sp.map.update()
        self.sp.replace_bottom(self.sp.map.surface)

    def draw_everything(self):
        self.canvas.draw(client.screen)
        self.sp.draw(client.screen)
        interface.Interface.draw_everything(self)

    def handle_event(self,event):
        if event.type == MOUSEBUTTONDOWN:
            if event.button == 1:
                thing = self.canvas.get_thing_at_position(*event.pos)
                if thing:
                    if self.selected_thing:
                        self.canvas.unselect_icon(self.selected_thing)
                        self.canvas.unselect_rect()
                    self.selected_thing = thing
                    if isinstance(self.selected_thing,things.Monster):
                        if sound.on:
                            random.choice([sounds["hmmmgh"],
                                           sounds["huargh"]]).play()
                    self.canvas.select_icon(thing)
                elif self.selected_thing:
                    self.canvas.start_rect(*event.pos)
        elif event.type == MOUSEBUTTONUP:
            if event.button == 1 and self.selected_thing and \
                   self.canvas.selected_rect:
                if self.canvas.selected_rect.w < 0:
                    self.canvas.selected_rect.x += self.canvas.selected_rect.w
                    self.canvas.selected_rect.w = abs(self.canvas.selected_rect.w)
                if self.canvas.selected_rect.h < 0:
                    self.canvas.selected_rect.y += self.canvas.selected_rect.h
                    self.canvas.selected_rect.h = abs(self.canvas.selected_rect.h)
                if isinstance(self.selected_thing,things.Monster):
                    server_stub.command_monster(self.selected_thing.id,
                                                self.canvas.selected_rect)
                    if sound.on:
                        random.choice([sounds["yesmaster"],
                                       sounds["yesmylord"]]).play()
                self.canvas.unselect_rect()
                self.canvas.unselect_icon(self.selected_thing)
                self.selected_thing = None
        if event.type == MOUSEMOTION:
            if event.buttons[0] and self.selected_thing is not None and \
                     self.canvas.selected_rect is not None:
                self.canvas.select_rect(*event.pos)
            if event.buttons[0] and self.selected_thing is None and \
                   self.canvas.selected_rect is None:
                self.canvas.move(-event.rel[0],-event.rel[1])
        return interface.Interface.handle_event(self,event)



