import pc_canvas
import pygame
from pygame.locals import *
from pgu import gui,text

iconsize = 64
textheight = 50
text_offset = 10

class Menu:
    def __init__(self, options):
        self.options = options

        self.surface = pygame.Surface((3*iconsize,(iconsize+textheight)*2))
        self.surface.fill(pc_canvas.panelcolor)

        self.subpanels = []

        x_pos = 0

        self.keymap = {}
        for option in self.options:
            for key in option.keys:
                self.keymap[key] = option
            temp_surface = pygame.Surface((iconsize,iconsize+textheight))
            temp_surface.fill(pc_canvas.panelcolor)
            temp_surface.blit(option.icon,(0,0))
            text = gui.App()
            table = gui.Table(x=text_offset,y=iconsize)
            table.add(gui.Label(option.action))
            text.init(widget=table,screen=temp_surface)
            table.rect.w = temp_surface.get_width() - table.rect.x
            table.rect.h = temp_surface.get_height() - table.rect.y
            text.paint(temp_surface)
            self.subpanels.append(pc_canvas.subPanel(temp_surface,
                                                    (x_pos+5,0,
                                                     temp_surface.get_width(),
                                                     temp_surface.get_height())))
            x_pos += iconsize

        self.sidepanel = None

    def activate(self,sidepanel):
        self.sidepanel = sidepanel
        for each in self.subpanels:
            self.sidepanel.add_panel(each,'middle3',center=False)
        global active_menu
        active_menu = self

    def deactivate(self):
        global active_menu
        active_menu = None
        for each in self.subpanels:
            self.sidepanel.remove_panel(each)

    def handle_mouse_event(self,pos):
        temp_sprite = pygame.sprite.Sprite()
        temp_sprite.rect = pygame.Rect(pos + (1,1))
        hits = pygame.sprite.spritecollide(temp_sprite,
                                           self.sidepanel.sprites,
                                           False)
        for i in range(len(self.subpanels)):
            if self.subpanels[i] in hits:
                return self.options[i].action
        return None

    # @return an action string
    # @return -1 if the menu has handled the event without an action
    # @return None if the menu is not interested in the event
    def handle_event(self, event):
        if event.type == KEYDOWN:
            return self.select(event.key)
        else:
            return None

    # @return an action string
    # @return -1 if the menu has handled the selection without an action
    # @return None if the
    def select(self, key):
        try:
            action = self.keymap[key].action
            if isinstance(action, Menu):
                self.deactivate()
                action.activate(self.sprites)
                return -1
            else:
                return action
        except KeyError:
            return None

class Option:
    def __init__(self, action, keys):
        # a string describing the action, or a submenu or parent menu
        self.action = action
        # keys that select the option
        self.keys = keys
        # icon that display the action to the player
        self.icon = menu_icons[action]
        
active_menu = None

menu_icons = {'fight': None,
              'heal': None,
              'fire': None
              }

for name in menu_icons.keys():
    menu_icons[name] = pygame.image.load('images/menu_'+name+'.bmp').convert()


