import socket
import string
import threading
import urlparse

import server
import things

debug = 0

class closed(Exception):
    def __init__(self, why):
        Exception.__init__(self, "Connection closed %s" % why)
        self.why = why

class Server:
    def __init__(self, address, port, nthreads):
        self.address = address
        self.port = port
        self.nthreads = nthreads

        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.bind((address, port))
        self.sock.listen(8)

        self.start_threads()

    class Thread(threading.Thread):
        pass

    def run(self):
        while 1:
            sock, addr = self.sock.accept()
            RequestHandler(self, sock, addr).handle()

    def start_threads(self):
        for i in range(self.nthreads):
            self.Thread(name = "server%d" % i, target = self.run).start()

class RequestHandler:
    def __init__(self, httpserver, sock, addr):
        self.server = httpserver
        self.sock = sock
        self.addr = addr
        
        self.req = ''

        self.clear_fields()

    def clear_fields(self):
        self.method = ''
        self.req_url = ''
        self.req_version = ''
        self.path = ''
        self.qs = ''
        self.query = {}
        self.headers = {}

    def handle(self):
        try:
            try:
                while 1:
                    self.clear_fields()
                    self.get_request()
                    Response(self).handle()
            except closed, exc:
                print exc
                self.sock.shutdown(socket.SHUT_RDWR)
                self.sock.close()
        except socket.error, exc:
            print exc
            # return and wait for next connect

    def get_request(self):
        self.get_request_line()
        self.get_headers()

    def get_request_line(self):
        nl = self.req.find('\r\n')
        while nl < 0:
            self.read()
            nl = self.req.find('\r\n')

        line = self.req[:nl]
        self.req = self.req[nl:]

        self.method, self.req_url, self.req_version = line.split()
        ignored, ignored, self.path, ignored, self.qs, ignored = \
              urlparse.urlparse(self.req_url)
        self.parse_qs()

    def get_headers(self):
        nlnl = self.req.find('\r\n\r\n')
        while nlnl < 0:
            self.read()
            nlnl = self.req.find('\r\n\r\n')

        headers = self.req[len('\r\n'):nlnl]  # skip newline from request line
        self.req = self.req[nlnl+len('\r\n\r\n'):]

        self.parse_headers(headers)

    def parse_headers(self, headers):
        if not headers: return
        for header in headers.split('\r\n'):
            name, value = header.split(':', 1)
            name = name.strip().lower()
            value = value.strip()
            if self.headers.has_key(name):
                self.headers[name].append(value)
            else:
                self.headers[name] = [value]

    def parse_qs(self):
        for pair in self.qs.split('&'):
            pair = pair.split('=', 1)
            if len(pair) == 1:
                pair = pair + ['',]
            key, value = pair
            if not key: continue
            if self.query.has_key(key):
                self.query[key].append(value)
            else:
                self.query[key] = [value]

    def read(self):
        text = self.sock.recv(1024)
        if not text: raise closed("by peer")
        self.req += text
        if debug: print "<<<", text

class Response:
    def __init__(self, request):
        self.request = request
        self.server = self.request.server
        self.sock = self.request.sock

        self.code = 200
        self.msg = 'OK'
        self.headers = {}
        self.headers['Connection'] = 'Keep-Alive'
        self.headers['Keep-Alive'] = 'timeout=600'
        self.wlines = []
        self.body = ''
        self.content_length = None

    def handle(self):
        self.do_GET()
        self.flush()

    def do_GET(self):
        server.global_lock.acquire()
        try: self.do_GET_locked()
        finally: server.global_lock.release()

    def do_GET_locked(self):
        if self.request.path == '/':
            self.wlines.append(front_page)
        elif self.request.path == '/echo.spy':
            self.do_echo()
        elif self.request.path == '/pc.spy':
            self.do_pc()
        elif self.request.path == '/dm.spy':
            self.do_dm()
        elif self.request.path == '/grid.spy':
            self.do_grid()
        elif self.request.path == '/state.spy':
            self.do_state()
        elif self.request.path == '/move.spy':
            self.do_move()
        elif self.request.path == '/chat.spy':
            self.do_chat()
        elif self.request.path == '/fight.spy':
            self.do_fight()
        elif self.request.path == '/heal.spy':
            self.do_heal()
        elif self.request.path == '/fire.spy':
            self.do_fire()
        elif self.request.path == '/command.spy':
            self.do_command()
        else:
            self.code, self.msg = 404, "Not Found"
            self.wlines.append(not_found)

    def flush(self):
        self.body = string.join(self.wlines, '')
        del self.wlines[:]
        self.content_length = len(self.body)
        
        self.headers['Content-Length'] = self.content_length
        
        self.send_response_line()
        self.send_headers()
        self.send_body()

    def send_response_line(self):
        self.send('HTTP/1.0 %d %s\r\n' % (self.code, self.msg))

    def send_headers(self):
        headers = []
        for header, value in self.headers.items():
            headers.append('%s: %s\r\n' % (header, value))
        headers.append('\r\n')

        self.send(string.join(headers, ''))

    def send_body(self):
        self.send(self.body)

    def send(self, text):
        if debug: print ">>>", text
        pos = 0
        while pos < len(text):
            pos += self.sock.send(text[pos:])

    ### Services

    # echo.spy

    def do_echo(self):
        self.wlines.append(repr(self.request.query))

    def do_pc(self):
        self.wlines.append(repr(server.map.get_pc(self.request.query["type"][0])))

    def do_dm(self):
        self.wlines.append(repr(server.map.get_dm()))

    def do_grid(self):
        self.wlines.append(repr(server.map.get_grid()))

    def do_state(self):
        self.wlines.append(repr(server.map.get_state()))

    def do_move(self):
        try:
            delta_x = int(self.request.query["x"][0])
            delta_y = int(self.request.query["y"][0])
        except (ValueError,KeyError):
            self.code, self.msg = 400, "Bad Request"
            self.wlines.append("Bad move parameters")
            return
        
        server.map.move(long(self.request.query["pid"][0]), delta_x, delta_y)
        # no body

    def do_command(self):
        pid = long(self.request.query["pid"][0])
        x = int(self.request.query["x"][0])
        y = int(self.request.query["y"][0])
        w = int(self.request.query["w"][0])
        h = int(self.request.query["h"][0])
        server.map.command_monster(pid,x,y,w,h)
        # no body

    def do_fight(self):
        player = server.map.things[long(self.request.query["pid"][0])]
        try:
            target = player.engaged()[0]
            self.wlines.append(repr([(target.id,
                                      player.attack(player.engaged()[0]))]))
        except IndexError:
            self.wlines.append(repr([]))

    def do_heal(self):
        level = 1 + server.player_crystals
        player = server.map.things[long(self.request.query["pid"][0])]
        
        if level == 1:
            player.heal(level, player)
        else:
            for delta_x in (-1, 0, 1):
                for delta_y in (-1, 0, 1):
                    x = player.position[0] + delta_x
                    y = player.position[1] + delta_y
                    thing = server.map.positions.get((x, y))
                    if isinstance(thing, things.Player):
                        player.heal(level, thing)

    def do_fire(self):
        level = 1 + server.player_crystals
        player = server.map.things[long(self.request.query["pid"][0])]

        if level == 1:
            try:
                target = player.engaged()[0]
                self.wlines.append(repr([(target.id,
                                          player.spell(level, player.engaged()[0]))]))
            except IndexError:
                self.wlines.append(repr([]))
        else:
            damages = []
            for monster in player.engaged():
                damages.append((monster.id, player.spell(level, monster)))
            self.wlines.append(repr(damages))
        
    def do_chat(self):
        server.setChat(self.request.query["pid"][0],self.request.query["chatmsg"][0])

        
front_page = """
<html>
<head>
<title>Welcome to Pencil Quest!</title>
</head>
<body>
<h1>Welcome to Pencil Quest!</h1>
</body>
</html>
"""

not_found = """
<html>
<head>
<title>Not Found</title>
</head>
<body>
The URL requested is not a valid page or service on this host.
</body>
</html>
"""
